local old_CloseSpecialWindows;
local NUMDKPROWS = 17;
local ScrollPaneBackdrop  = {
	bgFile = "Interface\\ChatFrame\\ChatFrameBackground",
	edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
	tile = true, tileSize = 16, edgeSize = 16,
	insets = { left = 3, right = 3, top = 5, bottom = 3 }
};
	
--- Toggles the display of the bidding window.
-- If the window is not constructed yet, it builds it.
function sdkp:ShowBids()
	local f = sdkp.newui;
	if not f then 
		-- build window
		local AceGUI = LibStub("AceGUI-3.0");
		f = AceGUI:Create("Frame");
		local window = f.frame;
		f:SetTitle("SDKP");
		-- I don't like the ace3 defaults
		window:SetWidth(575);
		window:SetHeight(400);
		f.sizer_se:Hide();
		f.sizer_e:Hide();
		f.sizer_s:Hide();
		f.statusbg:EnableMouse();
		f.statusbg:SetScript("OnMouseDown",f.title:GetScript("OnMouseDown"));
		f.statusbg:SetScript("OnMouseUp",f.title:GetScript("OnMouseUp"));
		
		local columns = {
			{ ["name"] = "Name", ["width"] = 113, ["defaultsort"] = "dsc", }, -- [1]
			{ ["name"] = "Main", ["width"] = 113, ["defaultsort"] = "dsc", }, -- [2]
			{ ["name"] = "TDA", ["width"] = 55,	["align"] = "RIGHT", ["sortnext"] = 4, }, -- [3]
			{ ["name"] = "SDA", ["width"] = 55, ["align"] = "RIGHT", ["sortnext"] = 5,}, -- [4]
			{ ["name"] = "NDA", ["width"] = 55, ["align"] = "RIGHT", ["sortnext"] = 7,}, -- [5]
			{ 
				["name"] = "Bid", 
				["width"] = 55, 
				["align"] = "RIGHT",
				["bgcolor"] = { ["r"] = 0.0, ["g"] = 0.0, ["b"] = 0.0, ["a"] = 0.5 },
				["sortnext"] = 3,
			}, -- [6]
			{ ["name"] = "DKP", ["width"] = 55, ["align"] = "RIGHT", ["sortnext"] = 1,}, -- [7]
		};
		self.BIDSCROLLFRAME = ScrollingTable:CreateST(columns, 12, 16, nil, window);
		self.BIDSCROLLFRAME.frame:SetPoint("TOP", window, "CENTER", 0, 46);
		self.BIDSCROLLFRAME.frame:SetPoint("LEFT", window, "LEFT", 15, 0);
		
		sdkp:UpdateScrollData();
		do 
			local value = self.db.profile.currentfilter;
			local filter = SDKP_FILTERS[SDKP_FILTERS_LIST[value]];
			if filter then 
				self.BIDSCROLLFRAME:SetFilter(filter);
			end
		end
		if sdkp.db.profile.currentsystem then 
			if SDKPDATA then
				self.BIDSCROLLFRAME:SetData(SDKPDATA[sdkp.db.profile.currentsystem] or {});
			else
				self.BIDSCROLLFRAME:SetData({});
			end
		end
		
		sdkp:BuildSlideFrame(window);	
		
		-- create the system dropdown
		local system = AceGUI:Create("Dropdown");
		system.frame:SetWidth(170);
		local list = {};
		for sys in pairs(SDKP_SYSTEMS or {}) do
			tinsert(list, sys);
		end
		system:SetList(list);
		system:SetText("");
		if self.db and self.db.profile.currentsystem then 
			system:SetText(self.db.profile.currentsystem);
		end
		system:SetLabel("Choose a system:");
		system:SetCallback("OnValueChanged",
			function(context, event, value)
				self.db.profile.currentsystem = context.list[value];
				sdkp:UpdateScrollData(); -- we should be doing this when the raid changes, but what the hell...
				self.BIDSCROLLFRAME:SetData(SDKPDATA[sdkp.db.profile.currentsystem]);
			end
		);
		f:AddChild(system);
		
		-- create the filter dropdown
		local filter = AceGUI:Create("Dropdown");
		local deletefilter = AceGUI:Create("Button");
		filter.frame:SetWidth(170);
		filter:SetList(SDKP_FILTERS_LIST);
		filter:SetLabel("Choose a display filter:");
		filter:SetCallback("OnValueChanged",
			function(context, event, value)
				sdkp:Debug("filter:OnValueChanged", event, value, context.list[value]);
				context:SetValue(value);
				self.db.profile.currentfilter = value;
				sdkp:UpdateScrollData(); -- we should be doing this when the raid changes, but what the hell...
				local filter = SDKP_FILTERS[context.list[value]];
				if filter then 
					self.BIDSCROLLFRAME:SetFilter(filter);
					deletefilter:SetDisabled(value < 3);
				else
					deletefilter:SetDisabled(true);
				end
			end
		);
		f.filtercontrol = filter;
		f:AddChild(filter);
		
		-- create the start bids button
		deletefilter.frame:SetWidth(75);
		deletefilter:SetText("Delete");
		deletefilter:SetCallback("OnClick", function(context, event)
			sdkp:EnqueueDialog(
				"|cffff7d0aSDKP|r\nAre you *sure* you want to delete this snapshot?",
				"Yes", 
				"No", 
				function ()	
					local value = filter.value;
					table.remove(sdkp.db.profile.snapshots, value - 2);
					
					SDKP_FILTERS[value] = nil;
					table.remove(SDKP_FILTERS_LIST, value);
					
					filter:SetList(SDKP_FILTERS_LIST);
					if SDKP_FILTERS[value] then
						filter:Fire("OnValueChanged", value);
					else
						filter:Fire("OnValueChanged", value - 1);
					end
				end
			);
			sdkp:ConfirmPopup();
		end);
		deletefilter.frame:SetParent(window);
		deletefilter.frame:SetPoint("BOTTOMLEFT", filter.frame, "BOTTOMRIGHT", 8, 0);
		f.deletefilter = deletefilter;
		
		-- and now, since acegui3 has retardedly few layout options(at time of writing), I must rape the framework.
		-- create edit box for bidding item
		local itemedit = CreateFrame("EditBox", "sdkpeditbox", window, "sdkpeditboxtemplate");
		itemedit:SetPoint("TOP", window, "TOP", 1, -42);
		
		local itemedropdown = CreateFrame("Button", "sdkplootdropdown", window, "sdkplootdropdowntemplate");
		itemedropdown:SetPoint("TOPLEFT", itemedit, "BOTTOMLEFT", 0, 10);
		itemedropdown:SetScript("OnShow", function() 
			sdkp:LootDropDown_OnShow(itemedropdown);
		end);
		itemedropdown:SetScript("OnClick", function() 
			sdkp:LootDropDown_OnClick(itemedropdown);
		end);
		itemedropdown:Hide(); itemedropdown:Show();
		
		-- create the start bids button
		local startbids = AceGUI:Create("Button");
		startbids.frame:SetWidth(130);
		startbids:SetText("Start Bids");
		startbids:SetCallback("OnClick",
			function(context, event)
				local item = itemedit:GetText();
				if context.text:GetText() == "Start Bids" then 
					self.BIDDING = true;
					context:SetText("Stop Bids");
					sdkp:SendRaidWarning("Bidding started for " .. item .. " ( full = " .. self:GetDkpForItem(item) .. " )");
					sdkp:SendRaidWarning("To bid: '/w " .. UnitName("player") .. " !bid X', where X is your bid.");
				else
					self.BIDDING = false;
					context:SetText("Start Bids");
					sdkp:SendRaidWarning("Bidding has ended for " .. item .. ".");
				end
			end
		);
		startbids.frame:SetParent(window);
		startbids.frame:SetPoint("TOPRIGHT", window, "TOPRIGHT", -20, -45);
		
		-- create the clear bids button
		local clearbids = AceGUI:Create("Button");
		clearbids.frame:SetWidth(130);
		clearbids:SetText("Clear Bids");
		clearbids:SetCallback("OnClick",
			function(context, event)
				itemedit:SetText("");
				self.db.profile.bids = {};
				self.BIDSCROLLFRAME:SortData();
			end
		);
		clearbids.frame:SetParent(window);
		clearbids.frame:SetPoint("TOPRIGHT", window, "TOPRIGHT", -20, -90);
		
		sdkp.newui = f;
	end
	
	-- hide the fucking window on escape.   Why this isn't part of the Frame widget boggles my mind.
	if not old_CloseSpecialWindows then
		old_CloseSpecialWindows = CloseSpecialWindows
		CloseSpecialWindows = function()
			if f.frame:IsVisible() then 
				f:Hide(); return true;
			else
				return old_CloseSpecialWindows();
			end
		end
	end
	f:Show();
end

-- Reconstruct the data for display.   Use if the data in table is out of date.
function sdkp:UpdateScrollData()
	if SDKP_NOUPDATE_ZONE[GetRealZoneText()] or IsActiveBattlefieldArena() then 
		self:Debug("We don't track here!", GetRealZoneText());
		return;
	end
	
	local system = sdkp.db.profile.currentsystem;
	if system then
		local i = 1;
		local nRaidMembers = GetNumRaidMembers();
		for raidIndex = 1, nRaidMembers do
			local raidMember, _, _, _, class = GetRaidRosterInfo(raidIndex);
			local raidMemberLower = raidMember:lower();
			local sdkp_player_index = SDKPPLAYER[raidMemberLower];
			
			-- Add Raid members to the dkp list, if not there.
			if not sdkp_player_index then 	
				local player_stub = { 
					["cols"] = { 
						{ ["value"] = raidMember, ["color"] = RAID_CLASS_COLORS[class:upper()]},
						{ ["value"] = "", },
						{ ["value"] = 0,},
						{ ["value"] = 0,},
						{ ["value"] = 0,},
						{ ["value"] = "", },
						{ ["value"] = sdkp.FUNC_DKP_TOTAL,},
					},
					["color"] = sdkp.COLOR_LOWPRIORITY,
				};
				player_stub.cols[7]["args"] = { player_stub.cols, 0, 0, 0 };
									
				-- Update with main's data
				local main = self:GetAltInfo(raidMember);
				if main ~= raidMember then 
					-- substitute in the main's data instead of the alt's data.
					local mdata = self:getPlayerData(main);
					if mdata then
						self:Debug("UpdateScrollData: Main for", raidMember, "=> ", main);
						player_stub.cols[2] = mdata.cols[1];
						for col = 3, #mdata.cols do
							player_stub.cols[col] = mdata.cols[col];
						end
					else
						self:Debug("UpdateScrollData: Main for", raidMember, "not found :(");
						self:Debug("UpdateScrollData:", raidMember, "not found, using the stub.");
					end
				end 
				for system, data in pairs(SDKPDATA) do 
					table.insert(data, player_stub);
					SDKPPLAYER[raidMemberLower] = #data;
				end
			end
		end
		self.BIDSCROLLFRAME:SortData();
	end
end

function sdkp:LootDropDown_OnShow(frame)
	UIDropDownMenu_Initialize(frame, function()
		if not sdkp.LOOT_WINDOW then 
			sdkp.LOOT_WINDOW = {}; 
		end
		
		for link in pairs(sdkp.LOOT_WINDOW) do
			UIDropDownMenu_AddButton({
				text = link,
				value = link,
				func = sdkp_LootDropDownSelect,
				notCheckable = nil,
				keepShownOnClick = nil,
			});
		end
	end);
end

function sdkp:LootDropDown_OnClick(frame)
	ToggleDropDownMenu(1, nil, frame, "sdkpeditbox", 0, 0);
end

function sdkp_LootDropDownSelect()
	getglobal("sdkpeditbox"):SetText(this.value);
end

function sdkp:BuildSlideFrame(parent)
	local SlidePaneBackdrop = {
		bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background",
		edgeFile = "Interface\\AddOns\\sdkp\\textures\\sdkp-slideout-border",
		tile = true, tileSize = 32, edgeSize = 32,
		insets = { left = 3, right = 3, top = 5, bottom = 3 }
	};
	local SlidePaneBackdropSlim = {
		bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background",
		edgeFile = "Interface\\AddOns\\sdkp\\textures\\sdkp-slideout-border-slim",
		tile = true, tileSize = 32, edgeSize = 32,
		insets = { left = 3, right = 3, top = 5, bottom = 3 }
	};
	
	local OPEN, CLOSED = 200, 20;
	-- slide frame
	local slideframe =  CreateFrame("Frame", "slideframe", parent);
	slideframe:EnableMouse();
	slideframe:SetFrameStrata("FULLSCREEN");
	slideframe:SetWidth(CLOSED);
	slideframe:SetHeight(350);
	slideframe:SetBackdrop(SlidePaneBackdropSlim);
	slideframe:SetPoint("LEFT", parent, "RIGHT", -8, 0);

	-- close button
	local closebutton = CreateFrame("Button", "closebutton", slideframe);
	closebutton:SetNormalTexture("Interface\\AddOns\\sdkp\\textures\\sdkp-slider-button-close");
	closebutton:SetFrameStrata("FULLSCREEN_DIALOG");
	closebutton:SetPoint("CENTER", slideframe, "RIGHT", -6, 0);
	closebutton:SetWidth(64);
	closebutton:SetHeight(128);
	closebutton:Hide();
	
	-- open button
	local openbutton = CreateFrame("Button", "openbutton", slideframe);
	openbutton:SetNormalTexture("Interface\\AddOns\\sdkp\\textures\\sdkp-slider-button-open");
	openbutton:SetFrameStrata("FULLSCREEN_DIALOG");
	openbutton:SetPoint("CENTER", slideframe, "RIGHT", -6, 0);
	openbutton:SetWidth(64);
	openbutton:SetHeight(128);

	-- loot detail
	local columns = {
		{
			["name"] = "Item", 
		  	["width"] = 32, 
		  	["DoCellUpdate"] = function(rowFrame, cellFrame, data, cols, row, realrow, column, fShow, ...)
		  		if fShow then 
			  		local cellData = data[realrow].cols[column];
			  		local itemTexture = GetItemIcon(cellData.id); 
					cellFrame:SetBackdrop( { bgFile = itemTexture });
				end				
		  	end,
		}, -- [1]
		{ ["name"] = "Winner", ["width"] = 68}, -- [2]
		{ ["name"] = "Cost", ["width"] = 40, ["align"] = "RIGHT"}, -- [3]
	};	
	self.LOOTDETAILFRAME = ScrollingTable:CreateST(columns, 9, 32, nil, slideframe);
	self.LOOTDETAILFRAME.frame:SetPoint("TOPLEFT", slideframe, "TOPLEFT", 8, -40);
	self.LOOTDETAILFRAME:SetData(self.db.profile.loot);
	
	self.LOOTTOOLTIP = CreateFrame("GameTooltip", "SDKPLOOTTOOLTIP", self.LOOTDETAILFRAME.frame, "GameTooltipTemplate");
	local sdkptooltip = self.LOOTTOOLTIP;  -- for closure
	self.LOOTDETAILFRAME:RegisterEvents({
		["OnEnter"] = function (rowFrame, cellFrame, data, cols, row, realrow, column, ...)
			if row and realrow and column == 1 then
				local rowData = data[realrow];
				sdkptooltip:SetOwner(cellFrame, "ANCHOR_NONE");
				sdkptooltip:SetPoint("TOPRIGHT", slideframe, "TOPLEFT", 0, 0);
				sdkptooltip:SetHyperlink(rowData.link);				
			end
		end,
		["OnLeave"] = function (rowFrame, cellFrame, data, cols, row, realrow, column, ...)
			if row and realrow and column == 1 then
				sdkptooltip:Hide();
			end
		end,
	});
	self.LOOTDETAILFRAME:Hide();
	
	local slide = function()
		local step = 0.2 * OPEN;
		local startwidth = slideframe:GetWidth();
		local isTextureSwapped = false;
		local direction = 1;
		if startwidth >= OPEN then 
			direction = -1;
		end
		sdkp:Debug(direction, startwidth);
		-- Hide contents
		self.LOOTDETAILFRAME:Hide();
		
		slideframe:SetScript("OnUpdate", function(time)
			local width = slideframe:GetWidth() + (step * direction);
			if width >= OPEN then 
				width = OPEN;
			elseif width <= CLOSED then 
				width = CLOSED;
			end
			slideframe:SetWidth(width+1);
			
			if not isTextureSwapped then -- normal border texture overlaps wierd with small frame sizes.
				if width < 40 and direction < 0 then 
					slideframe:SetBackdrop(SlidePaneBackdropSlim);
					isTextureSwapped = true;
				elseif width > 40 and direction > 0 then 
					slideframe:SetBackdrop(SlidePaneBackdrop);
					isTextureSwapped = true;
				end
			end
			
			-- we're done moving, finish up.
			local buttons = {["-1"] = openbutton, ["1"] = closebutton};
			if width == CLOSED or width == OPEN then 
				buttons[""..direction]:Show();
				buttons[""..(direction*-1)]:Hide();
				slideframe:SetScript("OnUpdate", nil);
				if width == OPEN then 
					-- show and refresh contents.
					self.LOOTDETAILFRAME:SortData();
					self.LOOTDETAILFRAME:Show();
				end
			end
		end);
	end;
	closebutton:SetScript("OnMouseUp", slide);
	openbutton:SetScript("OnMouseUp", slide);
end


-- UI action fucntions
do
	local texture = "|TInterface\\Buttons\\UI-Panel-MinimizeButton-Up:25:25:0:-1|t";
	sdkp.GetCloseButtonTexture = function()
		return texture;
	end	
end

